@@ -125,8 +125,9 @@ impl Parser {
                     "UPDATE" => Ok(self.parse_update()?),
                     "ALTER" => Ok(self.parse_alter()?),
                     "COPY" => Ok(self.parse_copy()?),
+                    "SET" => Ok(self.parse_set()?),
+                    "SHOW" => Ok(self.parse_show()?),
                     "START" => Ok(self.parse_start_transaction()?),
-                    "SET" => Ok(self.parse_set_transaction()?),
                     // `BEGIN` is a nonstandard but common alias for the
                     // standard `START TRANSACTION` statement. It is supported
                     // by at least PostgreSQL and MySQL.
@@ -285,16 +286,22 @@ impl Parser {
             self.expect_token(&Token::LParen)?;
             let partition_by = if self.parse_keywords(vec!["PARTITION", "BY"]) {
                 // a list of possibly-qualified column names
-                self.parse_expr_list()?
+                self.parse_comma_separated(Parser::parse_expr)?
             } else {
                 vec![]
             };
             let order_by = if self.parse_keywords(vec!["ORDER", "BY"]) {
-                self.parse_order_by_expr_list()?
+                self.parse_comma_separated(Parser::parse_order_by_expr)?
             } else {
                 vec![]
             };
-            let window_frame = self.parse_window_frame()?;
+            let window_frame = if !self.consume_token(&Token::RParen) {
+                let window_frame = self.parse_window_frame()?;
+                self.expect_token(&Token::RParen)?;
+                Some(window_frame)
+            } else {
+                None
+            };
 
             Some(WindowSpec {
                 partition_by,
@@ -313,38 +320,27 @@ impl Parser {
         }))
     }
 
-    pub fn parse_window_frame(&mut self) -> Result<Option<WindowFrame>, ParserError> {
-        let window_frame = match self.peek_token() {
-            Some(Token::Word(w)) => {
-                let units = w.keyword.parse::<WindowFrameUnits>()?;
-                self.next_token();
-                if self.parse_keyword("BETWEEN") {
-                    let start_bound = self.parse_window_frame_bound()?;
-                    self.expect_keyword("AND")?;
-                    let end_bound = Some(self.parse_window_frame_bound()?);
-                    Some(WindowFrame {
-                        units,
-                        start_bound,
-                        end_bound,
-                    })
-                } else {
-                    let start_bound = self.parse_window_frame_bound()?;
-                    let end_bound = None;
-                    Some(WindowFrame {
-                        units,
-                        start_bound,
-                        end_bound,
-                    })
-                }
-            }
-            Some(Token::RParen) => None,
-            unexpected => return self.expected("'ROWS', 'RANGE', 'GROUPS', or ')'", unexpected),
+    pub fn parse_window_frame(&mut self) -> Result<WindowFrame, ParserError> {
+        let units = match self.next_token() {
+            Some(Token::Word(w)) => w.keyword.parse::<WindowFrameUnits>()?,
+            unexpected => return self.expected("ROWS, RANGE, GROUPS", unexpected),
         };
-        self.expect_token(&Token::RParen)?;
-        Ok(window_frame)
+        let (start_bound, end_bound) = if self.parse_keyword("BETWEEN") {
+            let start_bound = self.parse_window_frame_bound()?;
+            self.expect_keyword("AND")?;
+            let end_bound = Some(self.parse_window_frame_bound()?);
+            (start_bound, end_bound)
+        } else {
+            (self.parse_window_frame_bound()?, None)
+        };
+        Ok(WindowFrame {
+            units,
+            start_bound,
+            end_bound,
+        })
     }
 
-    /// "CURRENT ROW" | ( (<positive number> | "UNBOUNDED") ("PRECEDING" | FOLLOWING) )
+    /// Parse `CURRENT ROW` or `{ <positive number> | UNBOUNDED } { PRECEDING | FOLLOWING }`
     pub fn parse_window_frame_bound(&mut self) -> Result<WindowFrameBound, ParserError> {
         if self.parse_keywords(vec!["CURRENT", "ROW"]) {
             Ok(WindowFrameBound::CurrentRow)
@@ -352,8 +348,7 @@ impl Parser {
             let rows = if self.parse_keyword("UNBOUNDED") {
                 None
             } else {
-                let rows = self.parse_literal_uint()?;
-                Some(rows)
+                Some(self.parse_literal_uint()?)
             };
             if self.parse_keyword("PRECEDING") {
                 Ok(WindowFrameBound::Preceding(rows))
@@ -597,7 +592,7 @@ impl Parser {
         } else {
             Expr::InList {
                 expr: Box::new(expr),
-                list: self.parse_expr_list()?,
+                list: self.parse_comma_separated(Parser::parse_expr)?,
                 negated,
             }
         };
@@ -769,7 +764,11 @@ impl Parser {
     #[must_use]
     pub fn parse_one_of_keywords(&mut self, keywords: &[&'static str]) -> Option<&'static str> {
         for keyword in keywords {
-            assert!(keywords::ALL_KEYWORDS.contains(keyword));
+            assert!(
+                keywords::ALL_KEYWORDS.contains(keyword),
+                "{} is not contained in keyword list",
+                keyword
+            );
         }
         match self.peek_token() {
             Some(Token::Word(ref k)) => keywords
@@ -808,6 +807,15 @@ impl Parser {
         }
     }
 
+    /// Bail out if the following tokens are not the expected sequence of
+    /// keywords, or consume them if they are.
+    pub fn expect_keywords(&mut self, expected: &[&'static str]) -> Result<(), ParserError> {
+        for kw in expected {
+            self.expect_keyword(kw)?;
+        }
+        Ok(())
+    }
+
     /// Consume the next token if it matches the expected token, otherwise return false
     #[must_use]
     pub fn consume_token(&mut self, expected: &Token) -> bool {
@@ -829,6 +837,21 @@ impl Parser {
         }
     }
 
+    /// Parse a comma-separated list of 1+ items accepted by `F`
+    pub fn parse_comma_separated<T, F>(&mut self, mut f: F) -> Result<Vec<T>, ParserError>
+    where
+        F: FnMut(&mut Parser) -> Result<T, ParserError>,
+    {
+        let mut values = vec![];
+        loop {
+            values.push(f(self)?);
+            if !self.consume_token(&Token::Comma) {
+                break;
+            }
+        }
+        Ok(values)
+    }
+
     /// Parse a SQL CREATE statement
     pub fn parse_create(&mut self) -> Result<Statement, ParserError> {
         if self.parse_keyword("TABLE") {
@@ -847,8 +870,7 @@ impl Parser {
         self.expect_keyword("TABLE")?;
         let table_name = self.parse_object_name()?;
         let (columns, constraints) = self.parse_columns()?;
-        self.expect_keyword("STORED")?;
-        self.expect_keyword("AS")?;
+        self.expect_keywords(&["STORED", "AS"])?;
         let file_format = self.parse_identifier()?.parse::<FileFormat>()?;
 
         self.expect_keyword("LOCATION")?;
@@ -872,11 +894,7 @@ impl Parser {
         // ANSI SQL and Postgres support RECURSIVE here, but we don't support it either.
         let name = self.parse_object_name()?;
         let columns = self.parse_parenthesized_column_list(Optional)?;
-        let with_options = if self.parse_keyword("WITH") {
-            self.parse_with_options()?
-        } else {
-            vec![]
-        };
+        let with_options = self.parse_with_options()?;
         self.expect_keyword("AS")?;
         let query = Box::new(self.parse_query()?);
         // Optional `WITH [ CASCADED | LOCAL ] CHECK OPTION` is widely supported here.
@@ -897,14 +915,10 @@ impl Parser {
         } else {
             return self.expected("TABLE or VIEW after DROP", self.peek_token());
         };
+        // Many dialects support the non standard `IF EXISTS` clause and allow
+        // specifying multiple objects to delete in a single statement
         let if_exists = self.parse_keywords(vec!["IF", "EXISTS"]);
-        let mut names = vec![];
-        loop {
-            names.push(self.parse_object_name()?);
-            if !self.consume_token(&Token::Comma) {
-                break;
-            }
-        }
+        let names = self.parse_comma_separated(Parser::parse_object_name)?;
         let cascade = self.parse_keyword("CASCADE");
         let restrict = self.parse_keyword("RESTRICT");
         if cascade && restrict {
@@ -922,12 +936,7 @@ impl Parser {
         let table_name = self.parse_object_name()?;
         // parse optional column list (schema)
         let (columns, constraints) = self.parse_columns()?;
-
-        let with_options = if self.parse_keyword("WITH") {
-            self.parse_with_options()?
-        } else {
-            vec![]
-        };
+        let with_options = self.parse_with_options()?;
 
         Ok(Statement::CreateTable {
             name: table_name,
@@ -1075,19 +1084,21 @@ impl Parser {
     }
 
     pub fn parse_with_options(&mut self) -> Result<Vec<SqlOption>, ParserError> {
-        self.expect_token(&Token::LParen)?;
-        let mut options = vec![];
-        loop {
-            let name = self.parse_identifier()?;
-            self.expect_token(&Token::Eq)?;
-            let value = self.parse_value()?;
-            options.push(SqlOption { name, value });
-            if !self.consume_token(&Token::Comma) {
-                break;
-            }
+        if self.parse_keyword("WITH") {
+            self.expect_token(&Token::LParen)?;
+            let options = self.parse_comma_separated(Parser::parse_sql_option)?;
+            self.expect_token(&Token::RParen)?;
+            Ok(options)
+        } else {
+            Ok(vec![])
         }
-        self.expect_token(&Token::RParen)?;
-        Ok(options)
+    }
+
+    pub fn parse_sql_option(&mut self) -> Result<SqlOption, ParserError> {
+        let name = self.parse_identifier()?;
+        self.expect_token(&Token::Eq)?;
+        let value = self.parse_value()?;
+        Ok(SqlOption { name, value })
     }
 
     pub fn parse_alter(&mut self) -> Result<Statement, ParserError> {
@@ -1113,8 +1124,7 @@ impl Parser {
     pub fn parse_copy(&mut self) -> Result<Statement, ParserError> {
         let table_name = self.parse_object_name()?;
         let columns = self.parse_parenthesized_column_list(Optional)?;
-        self.expect_keyword("FROM")?;
-        self.expect_keyword("STDIN")?;
+        self.expect_keywords(&["FROM", "STDIN"])?;
         self.expect_token(&Token::SemiColon)?;
         let values = self.parse_tsv()?;
         Ok(Statement::Copy {
@@ -1178,13 +1188,12 @@ impl Parser {
                         return parser_err!(format!("No value parser for keyword {}", k.keyword));
                     }
                 },
-                Token::Number(ref n) if n.contains('.') => match n.parse::<f64>() {
-                    Ok(n) => Ok(Value::Double(n.into())),
-                    Err(e) => parser_err!(format!("Could not parse '{}' as f64: {}", n, e)),
-                },
-                Token::Number(ref n) => match n.parse::<u64>() {
-                    Ok(n) => Ok(Value::Long(n)),
-                    Err(e) => parser_err!(format!("Could not parse '{}' as u64: {}", n, e)),
+                // The call to n.parse() returns a bigdecimal when the
+                // bigdecimal feature is enabled, and is otherwise a no-op
+                // (i.e., it returns the input string).
+                Token::Number(ref n) => match n.parse() {
+                    Ok(n) => Ok(Value::Number(n)),
+                    Err(e) => parser_err!(format!("Could not parse '{}' as number: {}", n, e)),
                 },
                 Token::SingleQuotedString(ref s) => Ok(Value::SingleQuotedString(s.to_string())),
                 Token::NationalStringLiteral(ref s) => {
@@ -1197,6 +1206,16 @@ impl Parser {
         }
     }
 
+    pub fn parse_number_value(&mut self) -> Result<Value, ParserError> {
+        match self.parse_value()? {
+            v @ Value::Number(_) => Ok(v),
+            _ => {
+                self.prev_token();
+                self.expected("literal number", self.peek_token())
+            }
+        }
+    }
+
     /// Parse an unsigned literal integer/long
     pub fn parse_literal_uint(&mut self) -> Result<u64, ParserError> {
         match self.next_token() {
@@ -1242,16 +1261,14 @@ impl Parser {
                 "TIMESTAMP" => {
                     // TBD: we throw away "with/without timezone" information
                     if self.parse_keyword("WITH") || self.parse_keyword("WITHOUT") {
-                        self.expect_keyword("TIME")?;
-                        self.expect_keyword("ZONE")?;
+                        self.expect_keywords(&["TIME", "ZONE"])?;
                     }
                     Ok(DataType::Timestamp)
                 }
                 "TIME" => {
                     // TBD: we throw away "with/without timezone" information
                     if self.parse_keyword("WITH") || self.parse_keyword("WITHOUT") {
-                        self.expect_keyword("TIME")?;
-                        self.expect_keyword("ZONE")?;
+                        self.expect_keywords(&["TIME", "ZONE"])?;
                     }
                     Ok(DataType::Time)
                 }
@@ -1333,22 +1350,17 @@ impl Parser {
         }
     }
 
-    /// Parse one or more identifiers with the specified separator between them
-    pub fn parse_list_of_ids(&mut self, separator: &Token) -> Result<Vec<Ident>, ParserError> {
-        let mut idents = vec![];
-        loop {
-            idents.push(self.parse_identifier()?);
-            if !self.consume_token(separator) {
-                break;
-            }
-        }
-        Ok(idents)
-    }
-
     /// Parse a possibly qualified, possibly quoted identifier, e.g.
     /// `foo` or `myschema."table"`
     pub fn parse_object_name(&mut self) -> Result<ObjectName, ParserError> {
-        Ok(ObjectName(self.parse_list_of_ids(&Token::Period)?))
+        let mut idents = vec![];
+        loop {
+            idents.push(self.parse_identifier()?);
+            if !self.consume_token(&Token::Period) {
+                break;
+            }
+        }
+        Ok(ObjectName(idents))
     }
 
     /// Parse a simple one-word identifier (possibly quoted, possibly a keyword)
@@ -1365,7 +1377,7 @@ impl Parser {
         optional: IsOptional,
     ) -> Result<Vec<Ident>, ParserError> {
         if self.consume_token(&Token::LParen) {
-            let cols = self.parse_list_of_ids(&Token::Comma)?;
+            let cols = self.parse_comma_separated(Parser::parse_identifier)?;
             self.expect_token(&Token::RParen)?;
             Ok(cols)
         } else if optional == Optional {
@@ -1424,7 +1436,7 @@ impl Parser {
     pub fn parse_query(&mut self) -> Result<Query, ParserError> {
         let ctes = if self.parse_keyword("WITH") {
             // TODO: optional RECURSIVE
-            self.parse_cte_list()?
+            self.parse_comma_separated(Parser::parse_cte)?
         } else {
             vec![]
         };
@@ -1432,7 +1444,7 @@ impl Parser {
         let body = self.parse_query_body(0)?;
 
         let order_by = if self.parse_keywords(vec!["ORDER", "BY"]) {
-            self.parse_order_by_expr_list()?
+            self.parse_comma_separated(Parser::parse_order_by_expr)?
         } else {
             vec![]
         };
@@ -1465,27 +1477,17 @@ impl Parser {
         })
     }
 
-    /// Parse one or more (comma-separated) `alias AS (subquery)` CTEs,
-    /// assuming the initial `WITH` was already consumed.
-    fn parse_cte_list(&mut self) -> Result<Vec<Cte>, ParserError> {
-        let mut cte = vec![];
-        loop {
-            let alias = TableAlias {
-                name: self.parse_identifier()?,
-                columns: self.parse_parenthesized_column_list(Optional)?,
-            };
-            self.expect_keyword("AS")?;
-            self.expect_token(&Token::LParen)?;
-            cte.push(Cte {
-                alias,
-                query: self.parse_query()?,
-            });
-            self.expect_token(&Token::RParen)?;
-            if !self.consume_token(&Token::Comma) {
-                break;
-            }
-        }
-        Ok(cte)
+    /// Parse a CTE (`alias [( col1, col2, ... )] AS (subquery)`)
+    fn parse_cte(&mut self) -> Result<Cte, ParserError> {
+        let alias = TableAlias {
+            name: self.parse_identifier()?,
+            columns: self.parse_parenthesized_column_list(Optional)?,
+        };
+        self.expect_keyword("AS")?;
+        self.expect_token(&Token::LParen)?;
+        let query = self.parse_query()?;
+        self.expect_token(&Token::RParen)?;
+        Ok(Cte { alias, query })
     }
 
     /// Parse a "query body", which is an expression with roughly the
@@ -1559,22 +1561,18 @@ impl Parser {
         if all && distinct {
             return parser_err!("Cannot specify both ALL and DISTINCT in SELECT");
         }
-        let projection = self.parse_select_list()?;
+        let projection = self.parse_comma_separated(Parser::parse_select_item)?;
 
         // Note that for keywords to be properly handled here, they need to be
         // added to `RESERVED_FOR_COLUMN_ALIAS` / `RESERVED_FOR_TABLE_ALIAS`,
         // otherwise they may be parsed as an alias as part of the `projection`
         // or `from`.
 
-        let mut from = vec![];
-        if self.parse_keyword("FROM") {
-            loop {
-                from.push(self.parse_table_and_joins()?);
-                if !self.consume_token(&Token::Comma) {
-                    break;
-                }
-            }
-        }
+        let from = if self.parse_keyword("FROM") {
+            self.parse_comma_separated(Parser::parse_table_and_joins)?
+        } else {
+            vec![]
+        };
 
         let selection = if self.parse_keyword("WHERE") {
             Some(self.parse_expr()?)
@@ -1583,7 +1581,7 @@ impl Parser {
         };
 
         let group_by = if self.parse_keywords(vec!["GROUP", "BY"]) {
-            self.parse_expr_list()?
+            self.parse_comma_separated(Parser::parse_expr)?
         } else {
             vec![]
         };
@@ -1604,6 +1602,74 @@ impl Parser {
         })
     }
 
+    pub fn parse_set(&mut self) -> Result<Statement, ParserError> {
+        let modifier = self.parse_one_of_keywords(&["SESSION", "LOCAL"]);
+        let variable = self.parse_identifier()?;
+        if self.consume_token(&Token::Eq) || self.parse_keyword("TO") {
+            let token = self.peek_token();
+            let value = match (self.parse_value(), token) {
+                (Ok(value), _) => SetVariableValue::Literal(value),
+                (Err(_), Some(Token::Word(ident))) => SetVariableValue::Ident(ident.as_ident()),
+                (Err(_), other) => self.expected("variable value", other)?,
+            };
+            Ok(Statement::SetVariable {
+                local: modifier == Some("LOCAL"),
+                variable,
+                value,
+            })
+        } else if variable == "TRANSACTION" && modifier.is_none() {
+            Ok(Statement::SetTransaction {
+                modes: self.parse_transaction_modes()?,
+            })
+        } else {
+            self.expected("equals sign or TO", self.peek_token())
+        }
+    }
+
+    pub fn parse_show(&mut self) -> Result<Statement, ParserError> {
+        if self
+            .parse_one_of_keywords(&["EXTENDED", "FULL", "COLUMNS", "FIELDS"])
+            .is_some()
+        {
+            self.prev_token();
+            self.parse_show_columns()
+        } else {
+            Ok(Statement::ShowVariable {
+                variable: self.parse_identifier()?,
+            })
+        }
+    }
+
+    fn parse_show_columns(&mut self) -> Result<Statement, ParserError> {
+        let extended = self.parse_keyword("EXTENDED");
+        let full = self.parse_keyword("FULL");
+        self.expect_one_of_keywords(&["COLUMNS", "FIELDS"])?;
+        self.expect_one_of_keywords(&["FROM", "IN"])?;
+        let table_name = self.parse_object_name()?;
+        // MySQL also supports FROM <database> here. In other words, MySQL
+        // allows both FROM <table> FROM <database> and FROM <database>.<table>,
+        // while we only support the latter for now.
+        let filter = self.parse_show_statement_filter()?;
+        Ok(Statement::ShowColumns {
+            extended,
+            full,
+            table_name,
+            filter,
+        })
+    }
+
+    fn parse_show_statement_filter(&mut self) -> Result<Option<ShowStatementFilter>, ParserError> {
+        if self.parse_keyword("LIKE") {
+            Ok(Some(ShowStatementFilter::Like(
+                self.parse_literal_string()?,
+            )))
+        } else if self.parse_keyword("WHERE") {
+            Ok(Some(ShowStatementFilter::Where(self.parse_expr()?)))
+        } else {
+            Ok(None)
+        }
+    }
+
     pub fn parse_table_and_joins(&mut self) -> Result<TableWithJoins, ParserError> {
         let relation = self.parse_table_factor()?;
 
@@ -1749,7 +1815,7 @@ impl Parser {
             let mut with_hints = vec![];
             if self.parse_keyword("WITH") {
                 if self.consume_token(&Token::LParen) {
-                    with_hints = self.parse_expr_list()?;
+                    with_hints = self.parse_comma_separated(Parser::parse_expr)?;
                     self.expect_token(&Token::RParen)?;
                 } else {
                     // rewind, as WITH may belong to the next statement's CTE
@@ -1812,16 +1878,7 @@ impl Parser {
     pub fn parse_update(&mut self) -> Result<Statement, ParserError> {
         let table_name = self.parse_object_name()?;
         self.expect_keyword("SET")?;
-        let mut assignments = vec![];
-        loop {
-            let id = self.parse_identifier()?;
-            self.expect_token(&Token::Eq)?;
-            let value = self.parse_expr()?;
-            assignments.push(Assignment { id, value });
-            if !self.consume_token(&Token::Comma) {
-                break;
-            }
-        }
+        let assignments = self.parse_comma_separated(Parser::parse_assignment)?;
         let selection = if self.parse_keyword("WHERE") {
             Some(self.parse_expr()?)
         } else {
@@ -1834,75 +1891,53 @@ impl Parser {
         })
     }
 
-    /// Parse a comma-delimited list of SQL expressions
-    pub fn parse_expr_list(&mut self) -> Result<Vec<Expr>, ParserError> {
-        let mut expr_list: Vec<Expr> = vec![];
-        loop {
-            expr_list.push(self.parse_expr()?);
-            if !self.consume_token(&Token::Comma) {
-                break;
-            }
-        }
-        Ok(expr_list)
+    /// Parse a `var = expr` assignment, used in an UPDATE statement
+    pub fn parse_assignment(&mut self) -> Result<Assignment, ParserError> {
+        let id = self.parse_identifier()?;
+        self.expect_token(&Token::Eq)?;
+        let value = self.parse_expr()?;
+        Ok(Assignment { id, value })
     }
 
     pub fn parse_optional_args(&mut self) -> Result<Vec<Expr>, ParserError> {
         if self.consume_token(&Token::RParen) {
             Ok(vec![])
         } else {
-            let args = self.parse_expr_list()?;
+            let args = self.parse_comma_separated(Parser::parse_expr)?;
             self.expect_token(&Token::RParen)?;
             Ok(args)
         }
     }
 
     /// Parse a comma-delimited list of projections after SELECT
-    pub fn parse_select_list(&mut self) -> Result<Vec<SelectItem>, ParserError> {
-        let mut projections: Vec<SelectItem> = vec![];
-        loop {
-            let expr = self.parse_expr()?;
-            if let Expr::Wildcard = expr {
-                projections.push(SelectItem::Wildcard);
-            } else if let Expr::QualifiedWildcard(prefix) = expr {
-                projections.push(SelectItem::QualifiedWildcard(ObjectName(prefix)));
+    pub fn parse_select_item(&mut self) -> Result<SelectItem, ParserError> {
+        let expr = self.parse_expr()?;
+        if let Expr::Wildcard = expr {
+            Ok(SelectItem::Wildcard)
+        } else if let Expr::QualifiedWildcard(prefix) = expr {
+            Ok(SelectItem::QualifiedWildcard(ObjectName(prefix)))
+        } else {
+            // `expr` is a regular SQL expression and can be followed by an alias
+            if let Some(alias) = self.parse_optional_alias(keywords::RESERVED_FOR_COLUMN_ALIAS)? {
+                Ok(SelectItem::ExprWithAlias { expr, alias })
             } else {
-                // `expr` is a regular SQL expression and can be followed by an alias
-                if let Some(alias) =
-                    self.parse_optional_alias(keywords::RESERVED_FOR_COLUMN_ALIAS)?
-                {
-                    projections.push(SelectItem::ExprWithAlias { expr, alias });
-                } else {
-                    projections.push(SelectItem::UnnamedExpr(expr));
-                }
-            }
-
-            if !self.consume_token(&Token::Comma) {
-                break;
+                Ok(SelectItem::UnnamedExpr(expr))
             }
         }
-        Ok(projections)
     }
 
-    /// Parse a comma-delimited list of ORDER BY expressions
-    pub fn parse_order_by_expr_list(&mut self) -> Result<Vec<OrderByExpr>, ParserError> {
-        let mut expr_list: Vec<OrderByExpr> = vec![];
-        loop {
-            let expr = self.parse_expr()?;
+    /// Parse an expression, optionally followed by ASC or DESC (used in ORDER BY)
+    pub fn parse_order_by_expr(&mut self) -> Result<OrderByExpr, ParserError> {
+        let expr = self.parse_expr()?;
 
-            let asc = if self.parse_keyword("ASC") {
-                Some(true)
-            } else if self.parse_keyword("DESC") {
-                Some(false)
-            } else {
-                None
-            };
-
-            expr_list.push(OrderByExpr { expr, asc });
-            if !self.consume_token(&Token::Comma) {
-                break;
-            }
-        }
-        Ok(expr_list)
+        let asc = if self.parse_keyword("ASC") {
+            Some(true)
+        } else if self.parse_keyword("DESC") {
+            Some(false)
+        } else {
+            None
+        };
+        Ok(OrderByExpr { expr, asc })
     }
 
     /// Parse a LIMIT clause
@@ -1910,16 +1945,13 @@ impl Parser {
         if self.parse_keyword("ALL") {
             Ok(None)
         } else {
-            self.parse_literal_uint()
-                .map(|n| Some(Expr::Value(Value::Long(n))))
+            Ok(Some(Expr::Value(self.parse_number_value()?)))
         }
     }
 
     /// Parse an OFFSET clause
     pub fn parse_offset(&mut self) -> Result<Expr, ParserError> {
-        let value = self
-            .parse_literal_uint()
-            .map(|n| Expr::Value(Value::Long(n)))?;
+        let value = Expr::Value(self.parse_number_value()?);
         self.expect_one_of_keywords(&["ROW", "ROWS"])?;
         Ok(value)
     }
@@ -1950,15 +1982,12 @@ impl Parser {
     }
 
     pub fn parse_values(&mut self) -> Result<Values, ParserError> {
-        let mut values = vec![];
-        loop {
-            self.expect_token(&Token::LParen)?;
-            values.push(self.parse_expr_list()?);
-            self.expect_token(&Token::RParen)?;
-            if !self.consume_token(&Token::Comma) {
-                break;
-            }
-        }
+        let values = self.parse_comma_separated(|parser| {
+            parser.expect_token(&Token::LParen)?;
+            let exprs = parser.parse_comma_separated(Parser::parse_expr)?;
+            parser.expect_token(&Token::RParen)?;
+            Ok(exprs)
+        })?;
         Ok(Values(values))
     }
 
@@ -1976,13 +2005,6 @@ impl Parser {
         })
     }
 
-    pub fn parse_set_transaction(&mut self) -> Result<Statement, ParserError> {
-        self.expect_keyword("TRANSACTION")?;
-        Ok(Statement::SetTransaction {
-            modes: self.parse_transaction_modes()?,
-        })
-    }
-
     pub fn parse_transaction_modes(&mut self) -> Result<Vec<TransactionMode>, ParserError> {
         let mut modes = vec![];
         let mut required = false;
@@ -2004,7 +2026,7 @@ impl Parser {
                 TransactionMode::AccessMode(TransactionAccessMode::ReadOnly)
             } else if self.parse_keywords(vec!["READ", "WRITE"]) {
                 TransactionMode::AccessMode(TransactionAccessMode::ReadWrite)
-            } else if required || self.peek_token().is_some() {
+            } else if required {
                 self.expected("transaction mode", self.peek_token())?
             } else {
                 break;
